# OTA Setup Mode File (ota_mitll.mod)
# Setup for Lincoln OTA CCID45
# 20060120 DGS

### Initialization stuff
mcbSeqEnable=0         # kill the sequencer run flag if set
mcbBkplnSlct=0         # 0=6 slot backplane
clkClkEnbl=1           # enable the clock board system clock
acqClkEnbl=1           # enable the acquisition board system clock
resetClk=1             # reset the clock board
clkPortData=0          # For bug in firmware, enable acces to the serial config stream 
resetCcd=1             # reset the acquisiton board

### OTA Configuration Stuff
arrayType=OTA
arrayID=CCID45
numOutputs=1
outputCfg=0            # One amplifier in lower left
pxlRows=4040           # number of active pixels in vertical direction 505 x 8 ((494 +11)x'OTA_Rows') ????
pxlCols=508            # number of active pixels in horizontal direction 480 + 28((480 + 28)x'OTA_Cols')
xPreScan=0             # number of prescan pixels
xPostScan=0            # 28 pixels = vertical street width (28x'OTA_Rows') but set to 0 for de-scrambler
yPostScan=0            # 11 pixels = horizontal street width (11x'OTA_Rows') but set to 0 for de-scrambler
xStart[]=1
yStart[]=1
mosaicRows=1
mosaicCols=8

### OTA Row / Column select parameters
OtSlctClkDivider=10     # Row/ Column select clock at max (100ns per edge). each unit increases delay by 50ns
OtDopPortEnable=1       # enable the DOP port on the clock board to use the clock port
OtGuideReg[7]=b00000000 # Specify the guide cells
OtGuideReg[6]=b00000000 # value of 1 selects the cell, LSB=column 7
OtGuideReg[5]=b00000000
OtGuideReg[4]=b00000000
OtGuideReg[3]=b00000000
OtGuideReg[2]=b00000000
OtGuideReg[1]=b00000000
OtGuideReg[0]=b00000000
OtDisableReg[7]=b00000000 # Specify the cells to disable
OtDisableReg[6]=b00000000
OtDisableReg[5]=b00000000
OtDisableReg[4]=b00000000
OtDisableReg[3]=b00000000
OtDisableReg[2]=b00000000
OtDisableReg[1]=b00000000
OtDisableReg[0]=b00000000

### Sequencer Attributes
Rows=505			# 494 rows + 11 post
Cols=508			# 480 cols + 28 post
RowBin=1
ColBin=1
RowSkip=0
ColSkip=0
OTA_Rows=8			# Number of OTA rows to read during science readout
Wipes=1				# number of times to clear the OTA before integration
mpuSeqClkDiv=0			# 50ns sequencer cycle time
contRun=0

### Data format
labSystem=1			# Set to 1 to use 'filename' as image prefix
intTime=0			# default exposure time
filename=mitll      # default filename prefix
imageFile=mitll     # Mnnn I can never remember which is the correct attribute to set this !!
rawPxlSize=4			# number of bytes in raw pixel value
finPxlSize=4			# number of bytes in finished pixel value
DoubleXfer=0			# 2 16-bit pixels per systran write
expVector=1			# 1=clear+readout, 2=clear only, 4=readout only
expMode=0			# mode of the exposure
processMode=0			# 0=Sep, 1=add_F1toF2
frmsPerRdOut=1			# number of readouts for one single exposure
captureMode=1			# 0=normal, 1=Sep, 2=Blockread
avNameSIze=32			# max character length of attribute name
avValueSize=32			# max character length of attribute value
avCommentSize=64		# max character length of attribute comment
rdOutTime[]=20000		# 20 second timeout on reads
finDataType=32			# 4 bytes = 32 bits

### Clock board settings
clkP1MuxSlct=0			# This directs P1 to the test port
clkP2MuxSlct=31			# This directs shutter to the test port
clkD5LedCntrl=1			# Turn off the lights

### Acquisition board settings
ccdCdsReg=x136      # quiescent state
ccdCdsEnbl=1        # enable CDS signals
ccdDopReg=0         # not used for this device
ccdXfrCount=8       # how many video channels are sent to the PAN
ccdRedirect[0]=0
ccdRedirect[1]=1
ccdRedirect[2]=2
ccdRedirect[3]=3
ccdRedirect[4]=4
ccdRedirect[5]=5
ccdRedirect[6]=6
ccdRedirect[7]=7
ccdPipeEnbl=1       # send data to PAN as acquired
ccdBurstEnbl=0      # send data to PAN upon request
ccdPipePriority=0   # only one acquisition board so highest priority
ccdDataWidth=3      # 1=16 LSBs, 2=16 MSBs, 3=18-bit data
ccdPowerDown[]=0    # All ADC turned on
ccdWarpMode[]=1     # Run at warp speed!
ccdGain[]=0         # Gain stage setting 0=1x, 1=2x

# Setup the telemetry attributes
ccdHVBiasTelMode[]=8  # Set telemetry mode to unipolar positive.
ccdHVBiasTelMode[31]=12  # Except last bank which is bipolar.
ccdHVBiasTelMode[30]=12
ccdHVBiasTelMode[29]=12
ccdHVBiasTelMode[28]=12
ccdHVBiasTelMode[27]=12
ccdHVBiasTelMode[26]=12
ccdHVBiasTelMode[25]=12
ccdHVBiasTelMode[24]=12
clkTelPwrDwn=0        # enable telemetry data for clock board
ccdTelShutdown=0      # enable telemetry data for ccd acq board


ccdHVBiasDacs[]=3.0   # and set all dacs safe at 3 volts
ccdDacClrMode2=0      # Dacs reset to a value of zero volts 
ccdDacClrMode1=0      
ccdDacClrMode0=0

### Video Input Settings
OS0AdcOfst=0.100
OS1AdcOfst=0.100
OS2AdcOfst=0.100
OS3AdcOfst=0.100
OS4AdcOfst=0.100
OS5AdcOfst=0.100
OS6AdcOfst=0.100
OS7AdcOfst=0.100

### Clock DAC settings
ShutterLowDac=0.0
ShutterHiDac=5.0
P1LowDac=2.0
P1HiDac=7.0
P2LowDac=2.0
P2HiDac=7.0
P3LowDac=2.0
P3HiDac=7.0
P4LowDac=2.0
P4HiDac=7.0
S1LowDac=-3.0
S1HiDac=6.0
S2LowDac=-3.0
S2HiDac=6.0
S3LowDac=-3.0
S3HiDac=6.0
SWLowDac=-3.0
SWHiDac=6.0
RGLowDac=-2.0
#### RGHiDac=10.0
RGHiDac=7.0          ### Temporary measure till we setup the config data properly

### Logic DAC Settings (using clocks instead of DOP)
D0LowDac=0.0
D0HiDac=7.5
D1LowDac=0.0
D1HiDac=7.5
D2LowDac=0.0
D2HiDac=7.5
CSEL0LowDac=0.0
CSEL0HiDac=7.5
CSEL1LowDac=0.0
CSEL1HiDac=7.5
CSEL2LowDac=0.0
CSEL2HiDac=7.5
CSEL3LowDac=0.0
CSEL3HiDac=7.5
CSEL4LowDac=0.0
CSEL4HiDac=7.5
CSEL5LowDac=0.0
CSEL5HiDac=7.5
CSEL6LowDac=0.0
CSEL6HiDac=7.5
CSEL7LowDac=0.0
CSEL7HiDac=7.5
RSEL0LowDac=0.0
RSEL0HiDac=7.5
RSEL1LowDac=0.0
RSEL1HiDac=7.5
RSEL2LowDac=0.0
RSEL2HiDac=7.5
RSEL3LowDac=0.0
RSEL3HiDac=7.5
RSEL4LowDac=0.0
RSEL4HiDac=7.5
RSEL5LowDac=0.0
RSEL5HiDac=7.5
RSEL6LowDac=0.0
RSEL6HiDac=7.5
RSEL7LowDac=0.0
RSEL7HiDac=7.5

### High Voltage Bias Settings (CCD ACQ Board)
RDDac=11.0
SCPDac=19.0
VDDHDac=16.0
VDDLDac=7.5
VSSDac=0.05     # Due to calibration constants, 0.05v is the lowest settable potential.
PSHDac=7.0      # on the HV bias channels.
PSLDac=2.0
GNDDac=0.05
OG1Dac=0.05
OG2Dac=0.05
SUBDac=0.05
DR0Dac=18.0			# Output drain biases
DR1Dac=18.0
DR2Dac=18.0
DR3Dac=18.0
DR4Dac=18.0
DR5Dac=18.0
DR6Dac=18.0
DR7Dac=18.0
SO0Dac=0.05			# Output source biases
SO1Dac=0.05
SO2Dac=0.05
SO3Dac=0.05
SO4Dac=0.05
SO5Dac=0.05
SO6Dac=0.05
SO7Dac=0.05

### Enable the Clocks and Biases
ccdHVBiasEnbl=1        # Enable bias supplies to detector
clkClockPortEnbl=1     # Enable clocks to detector
clkBiasEnbl=0          # No biases used on clock board
clkFBiasEnbl=0         # No fast biases used on clock board

### Configure the lights
mcbMpuSeqBusReq=1      # fire sequencer LED on front panel
mcbPixDataRdy=1        # fire pixel LED when data ready to send to PAN
clkBrdSel=1            # fire clock board LED when selected for read/write
clkDopReq=0            # fire clock board LED when DOP asks for write to clock port
clkDopAck=0            # fire clock board LED when clock port acknowledges DOP port request
ccdBrdSel=1            # fire acquisition board LED when selected for read/write
ccdCdsLoad=0           # fire acquisition board LED when CDS is loaded
ccdDataXfer=0          # fire acquisition board LED when data is being transferred
ccdCpldLedEnable=0     # switch off the cpld LEDs

### Bugs and features !! ##
resetCcd=1             # reset the acquisiton board again .. there is a hanging state on the cds or dop fsm.
 

