******************************************************
** PROJECT     : WIYN TEST DEVICE SEQUENCER CODE
** TITLE       : STA-1571 512x512 Single Cell non-OT CCD IN TEST DEWAR
**             : 
** VERSION     : 001
** DATE        : 5/2/2005
** AUTHOR      : DAVE SAWYER
** HISTORY     : VERSION 1 -- D, Sawyer - Initial sequencer based on STA0510 ver.2 
**	       : Version 2 -- DGS - Changed SW for clocking independent of S3
**				  - Also set INTEG_TIME as external variable (sta1571.mod file)
** DESCRIPTION : First test of STA device.
**             : COMPILE WITH ASM4 OR LATER.
** SIGNAL
** ASSIGNMENTS :
**             : D2      => CLK_OUT_00
**             : Rst     => CLK_OUT_01
**             : D0      => CLK_OUT_02
**             : SW      => CLK_OUT_03
**             : S3      => CLK_OUT_04
**             : CSEL    => CLK_OUT_08
**             : S2      => CLK_OUT_09
**             : RSEL    => CLK_OUT_10
**             : S1      => CLK_OUT_11
**             : P3      => CLK_OUT_12
**             : P2      => CLK_OUT_13
**             : P1      => CLK_OUT_14
**             : OTG     => CLK_OUT_16
**     
**
**
**********************
** TIMING CONSTANTS **
**********************
DEF #THETA_OVERLAP  10              * 10 us read parallel phase overlaps (us)
DEF #RESET_OVERLAP  10              * 10 us reset parallel phase overlap (us)
DEF #SWEEP_OVERLAP  2               * 2 sequencer clocks = 100ns (clk)
DEF #RST_TIME       10              * Delay for Reset on time    (clk) 500 ns
DEF #RST_DLY        10              * Delay for signal to settle (clk) 500 ns
DEF #INT_TIME       80              * Delay for integration time (clk) about 4 us
DEF #SER_OVERLAP    2               * Delay serial clock overlap (clk) about 100 ns
DEF #ODP_DLY	    10		    * Delay to allow logic switch (clk) about 10 us

***************************
** CLOCK BIT DEFINITIONS **
***************************
DEF #SHUTTER       B10000000000000000000000000000000
*
DEF #D0            B00000000000000000000000000000100  
DEF #D2            B00000000000000000000000000000001  
DEF #CSEL          B00000000000000000000000100000000
DEF #RSEL          B00000000000000000000010000000000
*
DEF #SW            B00000000000000000000000000001000 
DEF #Rst           B00000000000000000000000000000010  
*
DEF #S3            B00000000000000000000000000010000
DEF #S2            B00000000000000000000001000000000
DEF #S1            B00000000000000000000100000000000
*
DEF #P3            B00000000000000000001000000000000
DEF #P2            B00000000000000000010000000000000
DEF #P1            B00000000000000000100000000000000
*
DEF #OTG           B00000000000000010000000000000000 

** LOGIC SWITCHING PATTERNS
DEF #ODP_0	#RSEL OR #CSEL 		*No Cells addressed
DEF #ODP_1  	#RSEL OR #CSEL OR #D2
DEF #ODP_2  	#CSEL OR #D2
DEF #ODP_3  	#D2			*Latched Active Output
DEF #ODP_4  	#RSEL OR #D2

** PARALLEL CLOCK BIT PATTERNS for P2-P1-P3 clocking
** This is non-MPP clocking - parallel phase 1 high during integration
DEF #T_STATE_0     #P2                                  * T1=LO, T2=HI, T3=LO - STATIC CONDITION
DEF #T_STATE_1     #P1 OR #P2                           * T1=HI, T2=HI, T3=LO
DEF #T_STATE_2     #P1                                  * T1=HI, T2=LO, T3=LO
DEF #T_STATE_3     #P1 OR #P3                           * T1=HI, T2=LO, T3=HI
DEF #T_STATE_4     #P3                                  * T1=LO, T2=LO, T3=HI
DEF #T_STATE_5     #P2 OR #P3                           * T1=LO, T2=HI, T3=HI

** SERIAL CLOCK BIT PATTERNS for S1-S2-S1 clocking (charged transferred to S2 and S3)
DEF #S_STATE_0     #S2 OR #S3                     	
DEF #S_STATE_1     #S3                 
DEF #S_STATE_2     #S3 OR #S1         
DEF #S_STATE_3     #S1 		            
DEF #S_STATE_4     #S1 OR #S2      
DEF #S_STATE_5     #S2           

** PARALLEL CLOCKING VECTORS
DEF #PAR_0      #T_STATE_0 OR #S_STATE_0 OR #Rst OR #ODP_3
DEF #PAR_1      #T_STATE_1 OR #S_STATE_0 OR #Rst OR #ODP_3
DEF #PAR_2      #T_STATE_2 OR #S_STATE_0 OR #Rst OR #ODP_3
DEF #PAR_3      #T_STATE_3 OR #S_STATE_0 OR #Rst OR #ODP_3
DEF #PAR_4      #T_STATE_4 OR #S_STATE_0 OR #Rst OR #ODP_3
DEF #PAR_5      #T_STATE_5 OR #S_STATE_0 OR #Rst OR #ODP_3

** TRANSITION CLOCKING VECTORS (PAR TO SER)
DEF #TRN_0      #T_STATE_0 OR #S_STATE_1 OR #Rst
DEF #TRN_1      #T_STATE_0 OR #S_STATE_1 
DEF #TRN_2      #T_STATE_0 OR #S_STATE_2
DEF #TRN_3      #T_STATE_0 OR #S_STATE_3
DEF #TRN_4      #T_STATE_0 OR #S_STATE_4
DEF #TRN_5      #T_STATE_0 OR #S_STATE_5

** SERIAL CLOCKING VECTORS (output amp is active during serial clocking, D2=HI)
DEF #SER_0      #S_STATE_0 OR #T_STATE_0 OR #Rst OR #ODP_3
DEF #SER_1      #S_STATE_0 OR #T_STATE_0 OR #ODP_3
DEF #SER_2      #S_STATE_0 OR #T_STATE_0 OR #SW OR #ODP_3
DEF #SER_3      #S_STATE_1 OR #T_STATE_0 OR #SW OR #ODP_3
DEF #SER_4      #S_STATE_2 OR #T_STATE_0 OR #SW OR #ODP_3
DEF #SER_5      #S_STATE_3 OR #T_STATE_0 OR #ODP_3		*Video dump occurs here
DEF #SER_6      #S_STATE_4 OR #T_STATE_0 OR #ODP_3
DEF #SER_7      #S_STATE_5 OR #T_STATE_0 OR #ODP_3

DEF #SWP_0      #S_STATE_0 OR #T_STATE_0 OR #SW OR #Rst OR #ODP_3
DEF #SWP_1      #S_STATE_1 OR #T_STATE_0 OR #SW OR #Rst OR #ODP_3
DEF #SWP_2      #S_STATE_2 OR #T_STATE_0 OR #Rst OR #ODP_3
DEF #SWP_3      #S_STATE_3 OR #T_STATE_0 OR #Rst OR #ODP_3
DEF #SWP_4      #S_STATE_4 OR #T_STATE_0 OR #Rst OR #ODP_3
DEF #SWP_5      #S_STATE_5 OR #T_STATE_0 OR #Rst OR #ODP_3

** INTEGRATION AND INITIALIZATION CLOCKING VECTORS
DEF #INT_0      #T_STATE_0 OR #S_STATE_0 OR #Rst OR #ODP_0
DEF #SHUT_OPEN  #INT_0 OR #SHUTTER
DEF #SHUT_CLOSE #INT_0

*************************
** CDS BIT DEFINITIONS **
*************************
DEF #CDS_ADR    x0100                   * Channel address of CDS (0x100 = channel 0, 0x200 = channel 1, 0x300 = channel 0 and 1, etc.)
DEF #CDS_GBL    b1000000000000000       * Global address strobe for CTC and FP Trigger
DEF #CDS_CTC    b0000000000000001       * Trigger an ADC conversion cycle
DEF #CDS_INV    b0000000000000010       * Connect the Inverted the signal
DEF #CDS_NIN    b0000000000000100       * Connect the Non-Inverted the signal
DEF #CDS_INT    b0000000000001000       * Integrate
DEF #CDS_DCR    b0000000000010000       * Apply the DC Restore clamp
DEF #CDS_RST    b0000000000100000       * Reset the integrator
DEF #CDS_TFP    b0000000001000000       * Trigger the front panel test point

** CDS BIT PATTERNS
** GLOBAL FUNCTIONS
DEF #GLB_CTC    #CDS_GBL OR #CDS_CTC
DEF #GLB_TFP    #CDS_GBL OR #CDS_TFP
DEF #GLB_ALL    #CDS_GBL OR #CDS_CTC OR #CDS_TFP

** THESE VECTORS USED FOR NORMAL CDS OPERATION
DEF #CDS_0      #CDS_ADR OR #CDS_RST OR #CDS_NIN OR #CDS_INV OR #CDS_DCR
DEF #CDS_1      #CDS_ADR OR #CDS_RST OR #CDS_DCR OR #CDS_INV
DEF #CDS_2      #CDS_ADR OR #CDS_INV 
DEF #CDS_3      #CDS_ADR OR #CDS_INV OR #CDS_INT
DEF #CDS_4      #CDS_ADR OR #CDS_NIN
DEF #CDS_5      #CDS_ADR OR #CDS_NIN OR #CDS_INT 
DEF #CDS_6      #CDS_ADR OR #CDS_NIN 
DEF #CDS_7      #CDS_ADR OR #CDS_NIN OR #CDS_CTC

** THESE VECTORS USED FOR DC RESTORE ON OPERATION.
DEF #CDS_0C     #CDS_ADR OR #CDS_RST OR #CDS_NIN OR #CDS_INV OR #CDS_DCR
DEF #CDS_1C     #CDS_ADR OR #CDS_TFP OR #CDS_RST OR #CDS_NIN OR #CDS_INV OR #CDS_DCR
DEF #CDS_2C     #CDS_ADR OR #CDS_INV OR #CDS_DCR
DEF #CDS_3C     #CDS_ADR OR #CDS_INV OR #CDS_INT OR #CDS_DCR
DEF #CDS_4C     #CDS_ADR OR #CDS_INV OR #CDS_DCR
DEF #CDS_5C     #CDS_ADR OR #CDS_NIN OR #CDS_INV OR #CDS_DCR
DEF #CDS_6C     #CDS_ADR OR #CDS_NIN OR #CDS_DCR
DEF #CDS_7C     #CDS_ADR OR #CDS_NIN OR #CDS_INT OR #CDS_DCR
DEF #CDS_8C     #CDS_ADR OR #CDS_NIN OR #CDS_DCR
DEF #CDS_9C     #CDS_ADR OR #CDS_NIN OR #CDS_CTC OR #CDS_DCR

*******************************
** LOOP REGISTER DEFINITIONS **
*******************************
DEF #ROWS         0                 * NUMBER OF ROWS TO READOUT ((DET ROWS - SKIPROW) / ROWBIN)
DEF #COLS         1                 * NUMBER OF COLUMNS TO READOUT ((DET COLS - SKIPCOL) / COLBIN)
DEF #ROWBIN       2                 * BINNING FACTOR FOR ROWS
DEF #COLBIN       3                 * BINNING FACTOR FOR COLS
DEF #ROWSKIP      4                 * NUMBER OF ROWS TO SKIP FROM ORIGIN
DEF #COLSKIP      5                 * NUMBER OF COLS TO SKIP FROM ORIGIN
DEF #RESET_TIME   6                 * CLOCK TICKS TO HOLD INT RESET AND DC RESTORE TRUE
DEF #DWELL_TIME   7                 * CLOCK TICKS TO INTEGRATE SIGNAL FOR
DEF #HOLD_TIME    8                 * TIME TO WAIT FOR SIG SETTLE AFTER CDS CHANGES POLARITY 
DEF #WAIT_TIME    9                 * DWELL AFTER ISSUING CTC STROBE BEFORE BEGINING NEXT CDS SAMPLE
DEF #TST_CTC_CNT 12                 * NUMBER OF CTC STROBES TO GENERATE IN CDS, ADC, AND CLMPTST ROUTINES
DEF #TST_CTC_PRD 13                 * CLOCK TICKS TO WAIT BETWEEN CTC STROBES
DEF #WIPES	 14		    * NUMBER OF WIPES PRIOR TO INTEGRATION 
** 
**************************************************************
** JUMP FLAG DEFINITIONS - BIT POSITION IN COMMAND REGISTER **
**************************************************************
DEF #SYNC        1                  * Jump on sync bit high
DEF #ITC         2                  * Jump on Integration time expired
DEF #START       3                  * Jump on start exposure flag set
DEF #CCDREADOUT  4                  * STRVCTR =  1: DO COMPLETE CCD CLEAR - INTEGRATE - READ
DEF #CCDCLEAR    5                  * STRVCTR =  2: DO CCD CLEAR ONLY
DEF #CCDREAD     6                  * STRVCTR =  4: DO CCD READ ONLY
DEF #ADCTST      7                  * STRVCTR =  8: GENERATE CTC STROBES ONLY - NO CCD CLOCKS
DEF #CDSTST      8                  * STRVCTR = 16: TEST CDS ACTION AND GENERATE CTC STROBES - NO CCD CLOCKS
DEF #CLMPTST     9                  * STRVCTR = 32: TEST CDS ACTION, HOLD DC RESTORE ON, AND GENERATE CTC STROBES - NO CCD CLOCKS
DEF #VCTR_7     10                  * Jump on start vector bit value  64
DEF #VCTR_8     11                  * Jump on start vector bit value 128
DEF #CONT_RUN   12                  * USER BIT 1: LOOP INDEFINATELY ON TESTS
DEF #USR_2      13                  * Jump on user mode bit value 2
DEF #USR_3      14                  * Jump on user mode bit value 4
DEF #USR_4      15                  * Jump on user mode bit value 8
**
****************************
** EFR REGISTER CONSTANTS **
****************************
DEF #KILL_EXPFLG    x00000001       * Cancel start exposure flag using the EFR register
DEF #STRT_INTCNTR   x00000002       * Start the integration time counter using the EFR register
DEF #STOP_INTCNTR   x00000004       * Stop the integration time counter using the EFR register
DEF #LSR_AUTO_KILL  x00000100       * Enables automatic board select cancel after an LSR instruction
DEF #LSR_MAN_RESET  x00000200       * Disables automatic board select cancel. LSR will remain active until an LSR #DSLCT is issued
DEF #SET_SYNC_LOW   x00004000       * If enabled as master DHE, set sync bit low after sync delay
DEF #SET_SYNC_HIGH  x00008000       * If enabled as master DHE, set sync signal high immediately
DEF #KILLANDSYNC    x00004001       * Set sync low and kill start exposure flag
**
*****************************
** MODE REGISTER CONSTANTS **
*****************************
DEF #RESET       0                  * Mode register constant for board reset operation
DEF #READ        1                  * Mode register constant for read operation
DEF #WRITE16     2                  * Mode register constant for 16 bit write operation
DEF #WRITE32     3                  * Mode register constant for 32 bit write operation
**
**********************************
** LSR BOARD SELECT ASSIGNMENTS **
**********************************
DEF #DSLCT       0
DEF #MCB         1
DEF #CCDCLK      2
DEF #CCDACQ      8
**
** CCDCLK BOARD ADDRESS POINTERS
DEF #CLKPORT     x0000
DEF #FBIAS0123   x0001
DEF #FBIAS4567   x0002
**
** CCDACQ BOARD ADDRESS POINTERS
DEF #CDSREG      x0000
DEF #DOPREG      x0001
DEF #BRSTCMD     x0004
**
********************************
** PATTERN MEMORY DEFINITIONS **
********************************
** PATTERN MEMORY SEGMENTATION MAP
DEF #PATSEG_00      x0000             * Pattern memory origin - Initialization stuff
DEF #PATSEG_CLR     x0020             * Pattern memory origin - CCD CLEAR VECTORS
DEF #PATSEG_INIT    x0040             * Pattern memory origin - CCD READ INITIALIZATION
DEF #PATSEG_PAR     x0060             * Pattern memory origin - CCD PARALLEL SHIFT VECTORS
DEF #PATSEG_SER     x0080             * Pattern memory origin - CCD SERIAL SHIFT VECTORS
DEF #PATSEG_CDS     x00A0             * Pattern memory origin - CCD CDS OPERATION VECTORS
DEF #PATSEG_ODP     x00C0             * Pattern memory origin - CCD LOGIC SWITCH VECTORS

ORG PAT #PATSEG_00                    * ********* GENERAL CONSTANTS KEPT IN PAT MEM ********* 
PAT #KILL_EXPFLG    #KILL_EXPFLG      * We can use the same label name here because they are in different contexts
PAT #STRT_INTCNTR   #STRT_INTCNTR     * The definition earlier points to a data value. The label here points to
PAT #STOP_INTCNTR   #STOP_INTCNTR     * a pattern memory address

*ORG PAT #PATSEG_CLR                   * ********* CCD CLEAR VECTORS ********* 
*PAT #RST_0          #RST_0
*PAT #RST_1          #RST_1
*PAT #RST_2          #RST_2
*PAT #RST_3          #RST_3
*PAT #RST_4          #RST_4
*PAT #RST_5          #RST_5
*PAT #RST_6          #RST_6

ORG PAT #PATSEG_INIT                  * ********* CCD INTEGRATION AND READ INITIALIZATION ********* 
PAT #INT_0          #INT_0
PAT #SHUT_OPEN      #SHUT_OPEN
PAT #SHUT_CLOSE     #SHUT_CLOSE

ORG PAT #PATSEG_ODP                   * ********* CCD LOGIC SWITCH VECTORS ********* 
PAT #ODP_0          #ODP_0
PAT #ODP_1          #ODP_1
PAT #ODP_2          #ODP_2
PAT #ODP_3          #ODP_3
PAT #ODP_4          #ODP_4

ORG PAT #PATSEG_PAR                   * ********* CCD PARALLEL SHIFT VECTORS ********* 
PAT #PAR_0          #PAR_0
PAT #PAR_1          #PAR_1
PAT #PAR_2          #PAR_2
PAT #PAR_3          #PAR_3
PAT #PAR_4          #PAR_4
PAT #PAR_5          #PAR_5
PAT #TRN_0          #TRN_0
PAT #TRN_1          #TRN_1
PAT #TRN_2          #TRN_2
PAT #TRN_3          #TRN_3
PAT #TRN_4          #TRN_4
PAT #TRN_5          #TRN_5

ORG PAT #PATSEG_SER                   * ********* CCD SERIAL SHIFT VECTORS ********* 
PAT #SER_0          #SER_0
PAT #SER_1          #SER_1
PAT #SER_2          #SER_2
PAT #SER_3          #SER_3
PAT #SER_4          #SER_4
PAT #SER_5          #SER_5
PAT #SER_6          #SER_6
PAT #SER_7          #SER_7
PAT #SWP_0          #SWP_0
PAT #SWP_1          #SWP_1
PAT #SWP_2          #SWP_2
PAT #SWP_3          #SWP_3
PAT #SWP_4          #SWP_4
PAT #SWP_5          #SWP_5

ORG PAT #PATSEG_CDS                   * ********* CDS OPERATION VECTORS **********
PAT #GLB_CTC        #GLB_CTC          
PAT #GLB_TFP        #GLB_TFP        
PAT #GLB_ALL        #GLB_ALL                     
PAT #CDS_0          #CDS_0           
PAT #CDS_1          #CDS_1           
PAT #CDS_2          #CDS_2           
PAT #CDS_3          #CDS_3           
PAT #CDS_4          #CDS_4           
PAT #CDS_5          #CDS_5           
PAT #CDS_6          #CDS_6           
PAT #CDS_7          #CDS_7           
PAT #CDS_0C         #CDS_0C          
PAT #CDS_1C         #CDS_1C          
PAT #CDS_2C         #CDS_2C          
PAT #CDS_3C         #CDS_3C          
PAT #CDS_4C         #CDS_4C          
PAT #CDS_5C         #CDS_5C          
PAT #CDS_6C         #CDS_6C          
PAT #CDS_7C         #CDS_7C          
PAT #CDS_8C         #CDS_8C          
PAT #CDS_9C         #CDS_9C          

*************************************
* PROGRAM CODE SEGMENT STARTS HERE **
*************************************
** EXECUTIVE ROUTINE
#INIT   DMS 200                     * Allow the PAN to finish abusing the bus.
        CAL #INTEG                  * ESTABLISH THE INITIAL CONDITIONS

#EXEC   JCB #GOBABY #START          * IF START EXPOSURE FLAG SET
        JMP #EXEC                   * ELSE JUMP BACK TO EXEC

#GOBABY LPP #KILL_EXPFLG            * KILL THE START EXP FLAG PATTERN = 0x00000001
        LMR #WRITE32                * SET 32 BIT WRITE MODE
        LSR #MCB                    * WRITE TO THE EFR
        JCB #JRDOUT #CCDREADOUT     * DO CLEAR - INTEGRATE - READOUT OPERATION
        JCB #JCLEAR #CCDCLEAR       * DO CLEAR ONLY
        JCB #JREAD #CCDREAD         * DO READ ONLY
        JCB #LPTST #ADCTST          * RUN THE ADC TEST ROUTINE ONLY (GENERATE CTC STROBES)
        JCB #CDSTST #CDSTST         * RUN THE CDS TEST ROUTINE ONLY (RUN CDS, GENERATE CTC STROBES)
        JMP #EXEC                   * NO FLAGS SET, GO HOME.

** READOUT ROUTINE

#JRDOUT LMR #WRITE32
        LDA 3
        LSR 2                       * Bug fix !!
        LDA #CLKPORT
        LPP #ODP_0
        LSR #CCDCLK
	DUS #ODP_DLY
	CAL #LATCH
	LRB #WIPES
        CAL #CLEAR		    * Wipe the chip the number of times defined by WIPES
	LPE
	CAL #UNLATCH
        CAL #INTEG                  * PUT THE DETECTOR BACK INTO INTEGRATE MODE
        CAL #OPEN_S
        LPP #STRT_INTCNTR
        LSR #MCB                    * START INTEGRATION TIMER RUNNING
        NOP                         * A bug in the MPU .. needs time to reset the flag
        NOP

#RD1    JCB #RD2 #ITC
        JMP #RD1

#RD2    CAL #CLOSE_S
        LMR #WRITE32                * HERE WHEN ITC HAS REACHED REQUIRED TIME
        LPP #STOP_INTCNTR
        LSR #MCB                    * STOP INTEGRATION TIMER
        CAL #READ
        JCB #JRDOUT #CONT_RUN       * DO IT ALL AGAIN IF TOLD TO DO SO
        JMP #EXEC                   * CALL IT A DAY

#JCLEAR CAL #CLEAR
        JCB #JCLEAR #CONT_RUN       * DO IT ALL AGAIN IF TOLD TO DO SO
        JMP #EXEC

#JREAD  CAL #READ
        JCB #JREAD #CONT_RUN        * DO IT ALL AGAIN IF TOLD TO DO SO
        JMP #EXEC         
  
#OPEN_S LMR #WRITE32
        LDA #CLKPORT
        LPP #SHUT_OPEN
        LSR #CCDCLK
        RET
        
#CLOSE_S LMR #WRITE32
        LDA #CLKPORT
        LPP #SHUT_CLOSE
        LSR #CCDCLK
        RET

#CLEAR  CAL #RINIT                  * INITIALIZE FOR SERIAL READ
        CAL #SER_CLEAR              * SWEEP THE SERIAL REGISTER
        LRB #ROWS                   * LOOP FOR N ROWS
        CAL #PAR_SHIFT              * DO ROW XFERS FOR N BINS
*	CAL #P2S_TRANS		    * INITIALIZE THE SERIAL CLOCKS FOR SERIAL READ
        CAL #SER_CLEAR              * DO COLUMN CLEAR 
        LPE                         * END OF ROW LOOP
        RET
        
#INTEG  LMR #WRITE32                * SET 32 BIT WRITE MODE
        LDA #CLKPORT
        LPP #INT_0
        LSR #CCDCLK                 * Establish the clock phasing for integration
        LDA #CDSREG                 * Point to the CDS register on the CcdAcq board
        LPP #CDS_0
        LSR #CCDACQ                 * and set it down easy
        RET

#LATCH  LMR #WRITE32		    * LATCH THE OUTPUT LOGIC 
	LDA #CLKPORT
 	LPP #ODP_1
	LSR #CCDCLK
	DUS #ODP_DLY
        LPP #ODP_2
        LSR #CCDCLK
	DUS #ODP_DLY
        LPP #ODP_3
        LSR #CCDCLK
	DUS #ODP_DLY
	RET

#UNLATCH  LMR #WRITE32		    * UNLATCH THE OUTPUT LOGIC 
	LDA #CLKPORT
 	LPP #ODP_4
	LSR #CCDCLK
	DUS #ODP_DLY
 	LPP #ODP_0
	LSR #CCDCLK
	DUS #ODP_DLY
	RET

#READ   CAL #RINIT                  * INITIALIZE FOR READ
	CAL #LATCH		    * LATCH THE OUTPUT LOGIC
        CAL #SER_CLEAR              * SWEEP THE SERIAL REGISTER
        LRB #ROWS                   * LOOP FOR N ROWS
        CAL #PAR_SHIFT              * DO ROW XFERS FOR N BINS
*	CAL #P2S_TRANS		    * INITIALIZE THE SERIAL CLOCKS FOR SERIAL READ
        LRB #COLS                   * LOOP FOR N COLS
        CAL #SER_READ               * DO COLUMN READ FOR N BINS
        LPE                         * END OF COLUMN LOOP
        LPE                         * END OF ROW LOOP
	CAL #UNLATCH		    * UNLATCH THE OUTPUT LOGIC
        CAL #INTEG                  * PUT THE DETECTOR BACK INTO INTEGRATE MODE
        RET
        
#SER_CLEAR  LMR #WRITE32
        LDA #CLKPORT
        LRB #COLS                   * Do all the columns 
        LPP #SWP_0 
	LSR #CCDCLK
        DSC #SWEEP_OVERLAP
        LPP #SWP_1
        LSR #CCDCLK
        DSC #SWEEP_OVERLAP
        LPP #SWP_2
        LSR #CCDCLK
        DSC #SWEEP_OVERLAP
        LPP #SWP_3
        LSR #CCDCLK
        DSC #SWEEP_OVERLAP
        LPP #SWP_4
        LSR #CCDCLK
        DSC #SWEEP_OVERLAP
        LPP #SWP_5
        LSR #CCDCLK
        DSC #SWEEP_OVERLAP
        LPE
        RET

#SER_READ  LMR #WRITE32
	LDA #CLKPORT
        LPP #SER_0		*S1=LO S2=HI S3=HI SW=LO  RG=HI 
        LSR #CCDCLK                    
	DSC #RST_TIME		* HOLD FOR 500 ns	
	LPP #SER_1		*S1=LO S2=HI S3=HI SW=LO RG=LO 
        LSR #CCDCLK             
	DSC #SER_OVERLAP	
	LPP #SER_2		*S1=LO S2=HI S3=HI SW=HI RG=LO 
        LSR #CCDCLK             
	DSC #SER_OVERLAP	
	LDA #CDSREG
        LPP #CDS_1		* SET INV, DCR, RST
        LSR #CCDACQ
        LDA #CLKPORT
        LPP #SER_3		*S1=LO S2=LO S3=HI SW=HI RG=LO 
        LSR #CCDCLK             
	DSC #SER_OVERLAP	
	LPP #SER_4		*S1=HI S2=LO S3=HI SW=HI RG=LO 
        LSR #CCDCLK             
	DSC #SER_OVERLAP	
	LDA #CDSREG
	LPP #CDS_2		* SET INV, STOP DCR,RST
        LSR #CCDACQ
        DSC #RST_DLY		* SETTLING
        LPP #CDS_3		* SET INV, INT
        LSR #CCDACQ
*        DSC #INT_TIME		* INTEGRATE ON RESET
	LRB #DWELL_TIME
	DSC 1
	LPE
        LPP #CDS_4		* SET NIN, STOP INV,INT
        LSR #CCDACQ
        LDA #CLKPORT
        LPP #SER_5		* VIDEO DUMP OCCURS HERE S1=HI S2=LO S3=LO SW=LO RG=LO
        LSR #CCDCLK             
	DSC #SER_OVERLAP	
	DSC #RST_DLY		* SETTLING
	LDA #CDSREG
	LPP #CDS_5		* SET NIN,INT
	LSR #CCDACQ
*        DSC #INT_TIME		* INTEGRATE ON SIGNAL        
	LRB #DWELL_TIME
	DSC 1
	LPE
        LPP #CDS_6		* SET NIN, STOP INT
        LSR #CCDACQ
        LPP #CDS_7		* START CONVERT SET CTC, NIN
        LSR #CCDACQ
	LPP #CDS_0		* STOP CTC, SET INV,NIN,DCR,RST
        LSR #CCDACQ        
	LDA #CLKPORT
	LPP #SER_6		*S1=HI S2=HI S3=LO SW=LO RG=LO 
        LSR #CCDCLK             
	DSC #SER_OVERLAP	
	LPP #SER_7		*S1=LO S2=HI S3=LO SW=LO RG=LO 
        LSR #CCDCLK             
	DSC #SER_OVERLAP	
	RET

#RINIT  RET

#PAR_SHIFT  LMR #WRITE32
        LDA #CLKPORT
        LRB #ROWBIN                 * DO THIS FOR ROW BINNING FACTOR
        LPP #PAR_0
        LSR #CCDCLK                 * P1=HI P2=LO P3=LO
        DUS #THETA_OVERLAP        
	LPP #PAR_1
        LSR #CCDCLK                 * P1=HI P2=LO P3=HI
        DUS #THETA_OVERLAP
        LPP #PAR_2
        LSR #CCDCLK                 * P1=LO P2=LO P3=HI
        DUS #THETA_OVERLAP
        LPP #PAR_3
        LSR #CCDCLK                 * P1=LO P2=HI P3=HI
        DUS #THETA_OVERLAP
        LPP #PAR_4
        LSR #CCDCLK                 * P1=LO P2=HI P3=LO
        DUS #THETA_OVERLAP
        LPP #PAR_5
        LSR #CCDCLK                 * P1=HI P2=HI P3=LO
        DUS #THETA_OVERLAP
        LPP #PAR_0
        LSR #CCDCLK                 * P1=HI P2=LO P3=LO
        DUS #THETA_OVERLAP
        LPE
        RET

#P2S_TRANS  LMR #WRITE32
	LDA #CLKPORT
	LPP #TRN_0
	LSR #CCDCLK
	DSC #SER_OVERLAP
	LPP #TRN_1
	LSR #CCDCLK
	DSC #SER_OVERLAP
	LPP #TRN_2
	LSR #CCDCLK
	DSC #SER_OVERLAP
	LPP #TRN_3
	LSR #CCDCLK
	DSC #SER_OVERLAP
	LPP #TRN_4
	LSR #CCDCLK
	DSC #SER_OVERLAP
	LPP #TRN_5
	LSR #CCDCLK
	DSC #SER_OVERLAP
	RET

*
** TEST ROUTINES
*
#LPTST  LMR #WRITE32                * SET 32 BIT WRITE MODE
        LDA #CDSREG                 * DEVICE ADDRESS SET TO 0
        LPP #GLB_CTC                * PATTERN POINTER SET TO CORRECT CDS CODE WORD
        LRB #TST_CTC_CNT            * LOOP FOR NUMBER OF CTC STROBES REQUESTED
        LSR #CCDACQ                 * TRIGGER ADC
        LRB #TST_CTC_PRD            * WAIT FOR SPECIFIED NUMBER OF CLOCK CYCLES BETWEEN STROBES
        LPE                         * END OF WAIT LOOP
        LPE                         * END OF TEST LOOP
        JCB #LPTST #CONT_RUN        * DO IT ALL AGAIN IF TOLD TO DO SO
        JMP #EXEC

#CDSTST LMR #WRITE32                * SET 32 BIT WRITE MODE
        LDA #CDSREG                 * DEVICE ADDRESS SET TO 0
        LRB #TST_CTC_CNT            * LOOP FOR NUMBER OF PIXELS REQUESTED
        LPP #CDS_1                  * EXT_TRIGGER + RESTORE + RESET + INVERT + NONINV
        LSR #CCDACQ                 * OUTPUT REST STATE
        LRB #RESET_TIME
        LPE                         * RESET AND RESTORE TIME LOOP COUNTER
        LPP #CDS_2                  * INVERT = TRUE
        LSR #CCDACQ
        LPP #CDS_3
        LSR #CCDACQ                 * OUTPUT INTEGRATE ON RESET STATE
        LRB #DWELL_TIME
        LPE                         * INTEGRATION TIME LOOP COUNTER
        LPP #CDS_4
        LSR #CCDACQ                 * OUTPUT HOLD STATE
        LPP #CDS_5
        LSR #CCDACQ                 * OUTPUT CHANGE SIGNAL POLARITY STATE
        LRB #HOLD_TIME
        LPE                         * HOLD TIME LOOP COUNTER
        LPP #CDS_6
        LSR #CCDACQ                 * OUTPUT SELECT INVERTING STATE
        LPP #CDS_7
        LSR #CCDACQ                 * OUTPUT INTEGRATE ON VIDEO STATE
        LRB #DWELL_TIME
        LPE                         * INTEGRATION TIME LOOP COUNTER
*        LPP #CDS_8
*        LSR #CCDACQ                 * OUTPUT HOLD STATE
*        LRB #HOLD_TIME
*        LPE                         * HOLD TIME LOOP COUNTER
*        LPP #CDS_9
*        LSR #CCDACQ                 * OUTPUT CONVERT STATE
*        LRB #WAIT_TIME
*        LPE                         * WAIT UNTIL NEXT PIXEL
        LPE                         * END PIXEL LOOP
        LPP #CDS_0                  * RESTORE = TRUE
        LSR #CCDACQ                 * OUTPUT STATE_0
        JCB #CDSTST #CONT_RUN       * DO IT ALL AGAIN IF TOLD TO DO SO
        JMP #EXEC

#CLMPTST LMR #WRITE32               * SET 32 BIT WRITE MODE
        LDA #CDSREG                 * DEVICE ADDRESS SET TO 0
        LRB #TST_CTC_CNT            * LOOP FOR NUMBER OF PIXELS REQUESTED
        LPP #CDS_1C                 * EXT_TRIGGER + RESTORE + RESET + INVERT + NONINV
        LSR #CCDACQ                 * OUTPUT REST STATE
        LRB #RESET_TIME
        LPE                         * RESET AND RESTORE TIME LOOP COUNTER
        LPP #CDS_2C                 * INVERT = TRUE
        LSR #CCDACQ
        LPP #CDS_3C
        LSR #CCDACQ                 * OUTPUT INTEGRATE ON RESET STATE
        LRB #DWELL_TIME
        LPE                         * INTEGRATION TIME LOOP COUNTER
        LPP #CDS_4C
        LSR #CCDACQ                 * OUTPUT HOLD STATE
        LPP #CDS_5C
        LSR #CCDACQ                 * OUTPUT CHANGE SIGNAL POLARITY STATE
        LRB #HOLD_TIME
        LPE                         * HOLD TIME LOOP COUNTER
        LPP #CDS_6C
        LSR #CCDACQ                 * OUTPUT SELECT INVERTING STATE
        LPP #CDS_7C
        LSR #CCDACQ                 * OUTPUT INTEGRATE ON VIDEO STATE
        LRB #DWELL_TIME
        LPE                         * INTEGRATION TIME LOOP COUNTER
        LPP #CDS_8C
        LSR #CCDACQ                 * OUTPUT HOLD STATE
        LRB #HOLD_TIME
        LPE                         * HOLD TIME LOOP COUNTER
        LPP #CDS_9C
        LSR #CCDACQ                 * OUTPUT CONVERT STATE
        LRB #WAIT_TIME
        LPE                         * WAIT UNTIL NEXT PIXEL
        LPE                         * END PIXEL LOOP
        LPP #CDS_0C                 * PATTERN POINTER SET TO ORIGIN OF STATE SEQUENCE
        LDA #CDSREG                 * DEVICE ADDRESS SET TO 0
        LSR #CCDACQ                 * OUTPUT REST STATE
        JCB #CLMPTST #CONT_RUN      * DO IT ALL AGAIN IF TOLD TO DO SO
        JMP #EXEC

* END OF PROGRAM SEGMENT

